@rem = '--*-Perl-*--';
@rem = '
@echo off
if exist ..\common\ratlperl.exe (
    ..\common\ratlperl rereg.bat %*
    goto endofperl
)
if exist ratlperl.exe (
    .\ratlperl rereg.bat %*
    goto endofperl
)
if exist CQperl.exe (
    .\CQperl rereg.bat %*
    goto endofperl
)
echo ERROR: Unable to find ratlperl.exe or CQperl.exe.
goto endofperl
@rem ';

#
# Licensed Materials - Property of IBM and/or HCL
# Rational ClearQuest
# (C) Copyright IBM Corp. 1997, 2016 All Rights Reserved
# (C) Copyright HCL Technologies Ltd. 2016.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication, or
# disclosure is restricted by GSA ADP Schedule Contract with IBM
# Corporation.
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

#                              rereg.bat
#
# Reregister the ClearQuest installation in the current directory.
#
# Assumptions:
#   The common directory is in the PATH.
#   The current directory is the ClearQuest installation directory.
#

use warnings;
use strict;

use Cwd;
use File::Find;
use File::Spec;
use Win32::TieRegistry;


###############################################################################

sub Usage
{
    my (@msg) = @_;
    my $cmd = "rereg.bat";

    my $usage = <<EOF;
Usage: $cmd [-help] [-quiet] [-verbose]
            [-install] [-datadirect] [-servers] [-packages] [-unregister] [-all]
EOF

    if (@msg) {
        print "\n$cmd: ERROR: ", @msg, "\n";
        print <<EOF;
$usage
EOF
    }
    else {
        print <<EOF;

This utility re-registers the ClearQuest installed in the current directory.

$usage
Options:
  -h[elp]        -- display this help message
  -q[uiet]       -- minimal output
  -v[erbose]     -- maximum output
  -f[orce]       -- do not prompt to continue if installation appears broken
  -i[nstall]     -- register the ClearQuest installation directory
  -s[ervers]     -- register the ClearQuest COM servers
  -d[atadirect]  -- register the DataDirect drivers
  -p[ackages]    -- register packages
  -u[nregister]  -- unregister all packages (before doing -packages)
  -all           -- do all registrations (default)
EOF
    }
    exit 1;
}


###############################################################################
# Global variables and Utilities

our $opt_quiet = 0;
our $opt_force = 0;
our $opt_verbose = 0;

our $opt_all = 0;
our $opt_install = 0;
our $opt_servers = 0;
our $opt_datadirect = 0;
our $opt_packages = 0;
our $opt_unregister = 0;

our $ProductVersion = "";
our $ProductVersion_Label = "";

our @RegSvr32Opts = ('/c', '/s');
our $ComSvrOpts = '';
our $IntSvrOpts = '';


sub process_args
{
    my (@args) = @_;
    my @new_args;
    while (@args) {
        $_ = shift (@args);
        if (/^-/i) {
            if (/^-h(elp)?$/) {
                Usage();
            } elsif (/^-all$/) {
                $opt_all = 1;
            } elsif (/^-i(nstall)?$/) {
                $opt_install = 1;
            } elsif (/^-p(ackages)?$/) {
                $opt_packages = 1;
            } elsif (/^-u(nregister)?$/) {
                $opt_unregister = 1;
            } elsif (/^-q(uiet)?$/) {
                $opt_quiet = 1;
            } elsif (/^-f(orce)?$/) {
                $opt_force = 1;
            } elsif (/^-s(ervers)?$/) {
                $opt_servers = 1;
            } elsif (/^-v(erbose)?$/) {
                $opt_verbose = 1;
            } elsif (/^-d(atadirect)?$/) {
                $opt_datadirect = 1;
            } else {
                Usage ("Unknown option: $_");
            }
        } else {
            push (@new_args, $_);
        }
    }
    return @new_args;
}


sub System
{
    print join(' ', @_), "\n";
    my $rc = system(@_);
    return $rc;
}


sub regsvr32
{
    my $rc = System('regsvr32', @RegSvr32Opts, @_);
    if ($rc) {
        my ($hi, $lo) = ($rc >> 8, $rc & 0xff);
        warn("ERROR: Registration failed. Exit status: $hi\n");
    }
    return $rc;
}


###############################################################################
# Main

@ARGV = process_args(@ARGV);

if (scalar(@ARGV) > 0) {
    Usage("unrecognized extra arguments: " . join(", ", @ARGV));
    # Usage does not return.
}

# If no explicit registration options were specified, do all of them.
if (! ($opt_install || $opt_servers || $opt_datadirect ||
       $opt_packages || $opt_unregister )) {
    $opt_all = 1;
}

if ($opt_all) {
    $opt_install = 1;
    $opt_servers = 1;
    $opt_datadirect = 1;
    $opt_packages = 1;
    # Do NOT assume unregister packages; that requires an explicit option.
}

if ($opt_verbose) {
    @RegSvr32Opts = ();
    $ComSvrOpts = '-v';
    $IntSvrOpts = '/verbose';
}

if ($opt_quiet) {
    @RegSvr32Opts = ('/c', '/s');
    $ComSvrOpts = '';
    $IntSvrOpts = '';
}

# Get the path of the current directory.
my $Cwd = getcwd();
$Cwd =~ tr[/][\\];

# Add the current directory to the beginning of the path.
$ENV{PATH} = "$Cwd;$ENV{PATH}";


# Do a spot check on a few key files to see if the current directory looks
# like a ClearQuest install.  Only warn about this, since the registration
# attempt should do as much as possible even in a broken install, but give
# the user an option to stop without making any changes.

my @missing = ();
foreach my $needed ("installutil.exe",
                    "packageutil.exe",
                    "cqole.dll",
                    "cqmainttool.dll",
                    "cqinetlib.dll",
                    "cqintsvr11.exe") {
    unless (-e "$needed") {
        push @missing, $needed;
    }
}
if (scalar(@missing) > 0) {
    print "\n";
    print "WARNING: The current directory is $Cwd\n";
    print "WARNING: The ClearQuest installed here may be incomplete.\n";
    print "WARNING: Cannot find key files: " . join(", ", @missing) . ".\n";
    if (! $opt_force) {
        print "WARNING: If you continue, some registration steps may fail.\n";
        print "\n";
        print "Continue with registration [y/n]? ";
        my $answer = <STDIN>;
        if ($answer !~ /^y/i) {
            exit 1;
        }
    }
    else {
        print "WARNING: Forcing registration; some registration steps may fail.\n";
    }
    print "\n";
}

# Get the current version string, recognizing possible environmental override.
#
if (defined $ENV{CQ_REGISTRY_VERSION}) {
    $ProductVersion = $ENV{CQ_REGISTRY_VERSION};
    $ProductVersion_Label = ' (from CQ_REGISTRY_VERSION)';
}
else {
    # For development and testing purposes, the following provides a way to
    # use the CQ APIs to get the product version.  This is not normally used
    # since it relies on having a functioning CQPerlExt.pm, which itself
    # depends on the current installation being properly registered.  To use
    # this option, run the command like this:
    #
    #   cd <ClearQuest-home>
    #   ratlperl -s .\rereg.bat -UseClearQuestAPI=1 -- <args>
    #
    my $useAPI = 0;
    eval {
        # This 'eval' limits the scope of "no warnings".
        no warnings 'once';
        $useAPI = 1 if ($::UseClearQuestAPI);
    };

    if ($useAPI) {
        # Set CQ_HOME to make sure we get the right CQPerlExt.DLL.
        $ENV{CQ_HOME} = getcwd();

        require CQPerlExt;
        my $ClearQuest = CQClearQuest::Build();
        my $ProdInfo = $ClearQuest->CreateProductInfo();
        $ProductVersion = $ProdInfo->GetProductVersion();
        $ProductVersion_Label = ' (from ClearQuest API)';

        if ($ProductVersion eq '0.0.0') {
            print "ERROR: cannot register with local build product version $ProductVersion\n";
            exit (1);
        }
    }
    else {
        $ProductVersion = '8.0.0';
        $ProductVersion_Label = ' (from rereg)';
    }
}
if ($ProductVersion eq '') {
    $ProductVersion_Label =~ s/\(//;
    $ProductVersion_Label =~ s/\)//;
    print "ERROR: unable to determine product version$ProductVersion_Label\n";
    exit (1);
}

print <<EOF;
Updating the ClearQuest registry keys
  Install directory: $Cwd
  Product version:   ${ProductVersion}${ProductVersion_Label}
EOF


if ($opt_datadirect) {
    #
    # register the DataDirect ODBC drivers that are in ..\common\ODBC.
    #
    my $basedir = File::Spec->updir($Cwd);
    my $newdir = File::Spec->catdir($basedir, "common", "odbc");

    use Cwd 'abs_path';     # aka realpath()
    my $datadirectdir = abs_path($newdir);

    # RATLC01048953: rereg.bat corrupts ODBC driver registration
    # Convert slashes to backslashes
    $datadirectdir =~ s@/@\\@g;

    my $odbcinst_reg = "HKEY_LOCAL_MACHINE\\SOFTWARE\\ODBC\\ODBCINST.INI";

    # Reset the driver registry keys to point at the current directory.
    #
    print "\nRegistering the Rational DataDirect drivers in $datadirectdir ...\n";
    $Registry->{$odbcinst_reg .
                "\\IBM Rational Software DB2 Wire Protocol\\"} = {
        "APILevel" => "1",
        "ConnectFunctions" => "YYY",
        "DriverODBCVer" => "3.52",
        "FileUsage" => "0",
        "SQLLevel" => "1",
        "UsageCount" => "1",
        "CPTimeout" => "60",
        "Driver" => "$datadirectdir\\RKDB219.DLL",
        "Setup" => "$datadirectdir\\RKDB219.DLL",
        };
    $Registry->{$odbcinst_reg .
                "\\IBM Rational Software Oracle Wire Protocol\\"} = {
        "APILevel" => "1",
        "ConnectFunctions" => "YYY",
        "DriverODBCVer" => "3.52",
        "FileUsage" => "0",
        "SQLLevel" => "1",
        "UsageCount" => "1",
        "CPTimeout" => "60",
        "Driver" => "$datadirectdir\\RKORA19.DLL",
        "Setup" => "$datadirectdir\\RKORA19S.DLL",
        };
    $Registry->{$odbcinst_reg .
                "\\IBM Rational Software 7.1 DB2 Wire Protocol\\"} = {
        "APILevel" => "1",
        "ConnectFunctions" => "YYY",
        "DriverODBCVer" => "3.52",
        "FileUsage" => "0",
        "SQLLevel" => "1",
        "UsageCount" => "1",
        "CPTimeout" => "60",
        "Driver" => "$datadirectdir\\RKDB227.DLL",
        "Setup" => "$datadirectdir\\RKDB227.DLL",
        };
    $Registry->{$odbcinst_reg .
                "\\IBM Rational Software 7.1 Oracle Wire Protocol\\"} = {
        "APILevel" => "1",
        "ConnectFunctions" => "YYY",
        "DriverODBCVer" => "3.52",
        "FileUsage" => "0",
        "SQLLevel" => "1",
        "UsageCount" => "1",
        "CPTimeout" => "60",
        "Driver" => "$datadirectdir\\RKORA27.DLL",
        "Setup" => "$datadirectdir\\RKORA27.DLL",
        };
    $Registry->{$odbcinst_reg .
                "\\ODBC Drivers\\IBM Rational Software DB2 Wire Protocol"} =
                    "Installed";
    $Registry->{$odbcinst_reg .
                "\\ODBC Drivers\\IBM Rational Software Oracle Wire Protocol"} =
                    "Installed";
    $Registry->{$odbcinst_reg .
                "\\ODBC Drivers\\IBM Rational Software 7.1 DB2 Wire Protocol"} =
                    "Installed";
    $Registry->{$odbcinst_reg .
                "\\ODBC Drivers\\IBM Rational Software 7.1 Oracle Wire Protocol"} =
                    "Installed";
}

if ($opt_install) {
    #
    # Reset the install registry keys to point at the current directory.
    #
    print "\nRegistering the ClearQuest installation directory ...\n";
    $Registry->{'HKEY_LOCAL_MACHINE\\Software\\'} = {
        'Rational Software\\' => {
            'ClearQuest\\' => {
                '\\CurrentVersion' => $ProductVersion,
                $ProductVersion.'\\' => {
                    'Install\\' => {
                        '\\TARGETDIR' => $Cwd,
                        '\\DATADIR' => $Cwd,
                        '\\SRCDIR' => $Cwd,
                        'SqlAnywhere\\' => {
                            '\\BinPath' => $Cwd,
                        }
                    },
                },
            },
        },
    };

    # Invoke installutil to register the installation directory datapath.
    # It registers it under both HKEY_LOCAL_MACHINE and HKEY_CURRENT_USER.
    # Using installutil ensures that we are consistent with other uses of
    # installutil in the harness tests, which affect the same registry
    # entries.
    #
    System(qq(.\\installutil.exe registerdatapath "$Cwd")) &&
        warn("ERROR: Installation datapath registration failed. Exit status: ", $?>>8, "\n");
}

if ($opt_servers) {
    #
    # Reregister the COM objects.
    #
    print "\nRegistering the ClearQuest COM servers ...\n";

    regsvr32(qq(.\\cqole.dll));
    regsvr32(qq(.\\cqmainttool.dll));
    regsvr32(qq(.\\cqaxmform.dll));
    # remove state diagram for now
    # regsvr32(qq(.\\CQDDSShapes.dll));
    # regsvr32(qq(.\\CQStateDiagram.dll));
    regsvr32(qq(.\\CQICQDatastore.dll));
    regsvr32(qq(.\\cqemail.dll));
    regsvr32(qq(.\\cqmail.dll));
    regsvr32(qq(.\\cqmailinter.dll));
    regsvr32(qq(.\\cqratmail.dll));
    regsvr32(qq(.\\cqimportsvr.dll));
    regsvr32(qq(.\\cqinetlib.dll));
    regsvr32(qq(.\\cqaxrform.ocx));
    regsvr32(qq(.\\CQGCtrl.ocx));
    regsvr32(qq(.\\CQWkspTreeControl.ocx));
    # cqvssmgr requires visual source safe to register
    #   regsvr32(qq(.\\cqvssmgr.dll));
    # CQMSPInt requires microsoft project to register
    #   regsvr32(qq(.\\CQMSPInt.dll));

    System(qq(.\\cqintsvr11.exe $IntSvrOpts /regserver)) &&
        warn("ERROR: Registration failed. Exit status: ", $?>>8, "\n");
    System(qq(.\\cqintsvr10.exe $IntSvrOpts /regserver)) &&
        warn("ERROR: Registration failed. Exit status: ", $?>>8, "\n");
}

if ($opt_unregister) {
    #
    # Unregister all packages.  This is important to make sure the set of
    # packages are only from the current release.  Doing just the package
    # registration option only adds (or replaces) registration for packages
    # included with the current release.  It does not remove any packages that
    # are not known to the current release.  This could be a problem when the
    # current release is older than what was previously registered, and that
    # newer release had new packages or new versions of existing packages.
    #
    # This step should take no more than 3-4 minutes.
    #
    print "\nUnregister all versions of all packages ...\n";
    System(qq(.\\packageutil.exe registerpackage -all -all -unregister)) &&
        warn("ERROR: Unregistering all packages failed. Exit status: ", $?>>8, "\n");
}


if ($opt_packages) {
    #
    # Reregister the packages from the current directory.
    #
    # Note that using the "dir" command is generally faster than the perl
    # built-in 'find' command.  And invoking packageutil once with a "do"
    # script is much faster than invoking it once for each package version.
    # Note that the "do" script is generated with '-' on the front of each
    # line so that processing will continue even if one registration fails.
    #
    # This step should take no more than 3-4 minutes.

    print "\nFinding packages to register ...\n";

    my $pu_dofile = $ENV{TEMP} . "\\_cq_rereg_pu_commands_$$.txt";
    open my $PU_FILE, "> $pu_dofile"
        || die "ERROR: cannot open package registration script \"$pu_dofile\"";

    my $old_cwd = $Cwd;
    chdir("packages")
        || die "ERROR: cannot change to packages directory: $!\n";

    my $count = 0;
    open(FIND, "cmd.exe /C dir package.ini /b/s|")
        || die "Cannot run dir command to find package.ini files: $!\n";
    while (<FIND>) {
        chomp;
        s/\\package.ini$//;
        print $PU_FILE "-registerpackage \"$_\"\n";
        $count++;
    }
    close FIND;

    close $PU_FILE;

    print "Registering $count packages ...\n";

    my $last_package = "";
    my $okay = 0;
    my $fail = 0;

    open(PKGUTIL, "packageutil do $pu_dofile|")
         || die "Cannot run packageutil command on $pu_dofile: $!\n";
    while (<PKGUTIL>) {
        chomp;
        if ( /^>>> "registerpackage" "(.*)" $/ ) {
            # This is an echo of the command line from packageutil.  Yes, it
            # does have a space before the newline.  Save the package name in
            # case needed later for an error message, but do not print the
            # command itself.
            $last_package = $1;
            next;  # omit echo of command line
        }
        if ( /^\*\*\* / ) {
            $fail++;
            print "$_\n";
            print "ERROR: failed to register $last_package\n";

            # Unset the package name in case the message sequence is
            # unexpected and/or there is some other unrelated error.
            $last_package = "<unknown>";
            next;
        }
        if ( /^\+\+\+ / ) {
            $okay++;
            next if $opt_quiet;
        }

        # Print successful or unexpected output from packageutil.
        print "$_\n";
    }

    print "Registered $okay packages successfully\n";
    print "ERROR: Registration failed for $fail packages\n" if ($fail > 0);

    unlink $pu_dofile;
    chdir($old_cwd)
        || print "WARNING: unable to return to original directory: $!\n";
}

__END__

# This file is edited with vi tabs set to 4 chars wide (:set ts=4).  Keep
# this at the bottom of the file and EMACS will treat it the same way.
# Local Variables:
#   tab-width:4
#   indent-tabs-mode:nil
# End:

:endofperl
